/*
 *      Program do przegladania logow z ekg.
 *      Statystyki dla kontaktow.
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __CMD_LIST_HH__
#define __CMD_LIST_HH__

#include "command-base.h"
#include "texts.h"

#include <list>
#include <string>
using namespace std;


/**
 * @brief Komenda list
 */
class cmd_list_t : public command_base_t {
public:

	cmd_list_t() :  command_base_t("list", HELP_LIST_SHORT,
				HELP_LIST_LONG, true, true),
				_longestNick(0), _sorted(NONE)
	{}

	void run(const std::vector<std::string> & argv)
	{
		using namespace std;

		if (argv.size() != 1) {
			_info->print(LIST_WRONG_ARGS_NO, ALL_ERROR_FMT);
			return;
		}

		// jesli dla wszystkich
		if (!argv[0].compare(0, 2, "-a") || !argv[0].compare(0, 2, "-m") ||
		    !argv[0].compare(0, 2, "-c") || !argv[0].compare(0, 2, "-s"))
		{
			printAll(argv[0]);
			return;
		}

		// statystyki dla uina
		list<UinData>::iterator i = _data.begin();
		while (i!=_data.end() && i->nick != argv[0])
			++i;

		if (i == _data.end()) {
			_info->print(LIST_WRONG_NICK + argv[0], ALL_ERROR_FMT);
			return;
		}

		UinData & data = *i;
		string tmp = LIST_1 + argv[0] + LIST_2 + text_cast(data.uin) + LIST_3;
		tmp += LIST_4 + text_cast(data.msgs) + LIST_5;
		tmp += LIST_6 + text_cast(data.chats) + LIST_7;
		tmp += LIST_8 + text_cast(data.stats) + LIST_9;
		_info->print(tmp, ALL_INFO_FMT);

	} // run()

	// funkcje nalezy wywolac z zewnatrz
	void prepareCompl()
	{
		// lepiej zeby nie bylo nickow dluszych niz 30 znakow
		//                      012345678901234567890123456789012345678901234567890
		static string spaces = "                                                   ";


		_completition.clear();
		for (winlist_t::const_iterator i = _nicks->begin();
			i!=_nicks->end(); ++i)
		{
			_completition.push_back(i.str());
		}
		_completition.push_back("-a");
		_completition.push_back("-m");
		_completition.push_back("-c");
		_completition.push_back("-s");

		// od razu przygotujemy dane o kontaktach - w ten sposob
		// wszystko wywolane bedzie tylko raz przy ladowaniu listy
		// kontaktow, a nie przy kazdym wywolaniu komendy
		for (log_iterator i=_db->begin(); i!=_db->end(); ++i) {
			UinLog & log = i->second;
			if (_longestNick < log.nick().size())
				_longestNick = log.nick().size();

			UinData ud(log.nick(), log.uin(), log.msgCount(),
					log.size(), log.statusCount());

			ud.descr = LIST_ALL_1 + log.nick() + LIST_ALL_2;
			ud.descr += spaces.substr(0, 30-log.nick().size());
			string tmp = text_cast(ud.msgs);
			ud.descr += tmp + LIST_ALL_3 + spaces.substr(0,7-tmp.size());
			tmp = text_cast(ud.chats);
			ud.descr += tmp + LIST_ALL_4 + spaces.substr(0,7-tmp.size());
			tmp = text_cast(ud.stats);
			ud.descr += tmp + spaces.substr(0,7-tmp.size());

			_data.push_back(ud);
		}
		
	}

private:
	void printAll(const std::string & str)
	{
		if (!str.compare(0, 2, "-a") && _sorted != NICKS) {
			_data.sort(UinDataSortNicks());
			_sorted = NICKS;
		} else if (!str.compare(0, 2, "-m") && _sorted != MSGS) {
			_data.sort(UinDataSortMsgs());
			_sorted = MSGS;
		} else if (!str.compare(0, 2, "-c") && _sorted != CHATS) {
			_data.sort(UinDataSortChats());
			_sorted = CHATS;
		} else if (!str.compare(0, 2, "-s") && _sorted != STATS) {
			_data.sort(UinDataSortStats());
			_sorted = STATS;
		}

		_info->print(LIST_ALL_HEADER, ALL_INFO_FMT);
		for (list<UinData>::const_iterator i=_data.begin(); i!=_data.end(); ++i) {
			_info->print(i->descr, ALL_INFO_FMT);
		}
	}

	// pomocnicze
	struct UinData {
		UinData(string e, int a, int b, int c, int d)
			: uin(a), msgs(b), chats(c), stats(d), nick(e) {}
		int uin, msgs, chats, stats;
		string nick, descr;
	};

	struct UinDataSortNicks {
		bool operator () (const UinData & a, const UinData & b)
		{ return a.descr < b.descr; }
	};

	struct UinDataSortMsgs {
		bool operator () (const UinData & a, const UinData & b)
		{ return a.msgs > b.msgs; }
	};

	struct UinDataSortChats {
		bool operator () (const UinData & a, const UinData & b)
		{ return a.chats > b.chats; }
	};

	struct UinDataSortStats {
		bool operator () (const UinData & a, const UinData & b)
		{ return a.stats > b.stats; }
	};

	list<UinData> _data;
	unsigned int _longestNick;

	enum sorted { NICKS, MSGS, CHATS, STATS, NONE } _sorted;
};

#endif /* __CMD_STATS_HH__ */
